@@ -32,6 +32,7 @@ fit_hbd_model_parametric = function(tree,
 
 	# basic input control
 	if((Ntips<2) || (Nnodes<2)) return(list(success = FALSE, error="Tree is too small"));
+	if((!is.null(age_grid)) && (age_grid[1]>tail(age_grid,1))) age_grid = rev(age_grid); # avoid common errors where age_grid is in reverse order
 	if(is.null(oldest_age)) oldest_age = root_age;
 	if(Ntrials<1) return(list(success = FALSE, error = sprintf("Ntrials must be at least 1")))
 	if(!(condition %in% c("stem","crown","none"))) return(list(success = FALSE, error = sprintf("Invalid condition option '%s'; expected either 'crown', or 'stem' or 'none' (not recommended)",condition)))
@@ -92,7 +93,6 @@ fit_hbd_model_parametric = function(tree,
 	
 		
 	# determine which parameters are to be fitted
-	# convention: parameters are indexed as follows: [lambda[], mu[], rho]
 	fitted_params	= which(is.na(param_values))
 	fixed_params	= which(!is.na(param_values))
 	NFP				= length(fitted_params);
@@ -121,10 +121,10 @@ fit_hbd_model_parametric = function(tree,
 		params = param_values; params[fitted_params] = fparam_values * param_scale[fitted_params];
 		if(any(is.nan(params)) || any(is.infinite(params))) return(Inf); # catch weird cases where params become NaN
 		if(!is.null(param_names)) names(params) = param_names;
-		lambdas = lambda(age_grid,params)
-		mus 	= mu(age_grid,params)
-		rho 	= rho(params)
-		if(!(all(is.finite(lambdas)) && all(is.finite(mus)) && is.finite(rho))) return(Inf); # catch weird cases where lambda/mu/rho become NaN
+		lambdas 	= lambda(age_grid,params)
+		mus 		= mu(age_grid,params)
+		input_rho 	= rho(params)
+		if(!(all(is.finite(lambdas)) && all(is.finite(mus)) && is.finite(input_rho))) return(Inf); # catch weird cases where lambda/mu/rho become NaN
 		if(length(age_grid)==1){
 			# while age-grid has only one point (i.e., lambda & mu are constant over time), we need to provide a least 2 grid points to the loglikelihood calculator, spanning the interval [0,oldest_age]
 			input_age_grid 	= c(0,oldest_age);
@@ -137,7 +137,7 @@ fit_hbd_model_parametric = function(tree,
 		}
 		results = get_HBD_model_loglikelihood_CPP(	branching_ages		= sorted_node_ages,
 													oldest_age			= oldest_age,
-													rarefaction			= rho,
+													rarefaction			= input_rho,
 													age_grid 			= input_age_grid,
 													lambdas 			= input_lambdas,
 													mus 				= input_mus,
